# Copyright 2014-2016 A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

import six

from acos_client.v30 import base


class SLBCommon(base.BaseV30):

    url_prefix = "/slb/common"

    def _underscore_to_dash(self, val):
        rv = val.replace("_", "-")
        return rv

    def create(self, **kwargs):
        params = {"common": {}}
        for k, v in six.iteritems(kwargs):
            params["common"][self._underscore_to_dash(k)] = v
        kwargs = {}

        return self._post(self.url_prefix, params, **kwargs)



# Copyright 2014-2016, Omkar Telee, A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client import client
import acos_client.errors as acos_errors
import responses


HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/axapi/v3'.format(HOSTNAME)
AUTH_URL = '{}/auth'.format(BASE_URL)
AFLEX_NAME = 'test1'
CREATE_URL = '{}/file/aflex/'.format(BASE_URL)
OBJECT_URL = '{}/file/aflex/{}'.format(BASE_URL, AFLEX_NAME)


class TestAFlex(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '30', 'fake_username', 'fake_password')

    @mock.patch('acos_client.v30.slb.aflex_policy.AFlexPolicy.get')
    @responses.activate
    def test_aflex_create(self, mocked_get):
        mocked_get.side_effect = acos_errors.NotFound
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        filename = "testaflexpolicy"
        script = "when RULE_INIT{ change }"
        size = len(script.encode('utf-8'))
        action = "import"

        resp = self.client.slb.aflex_policy.create(filename, script, size, action)
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertIn(filename, responses.calls[1].request.body.decode('UTF-8'))
        self.assertIn(action, responses.calls[1].request.body.decode('UTF-8'))

    @mock.patch('acos_client.v30.slb.aflex_policy.AFlexPolicy.get')
    @responses.activate
    def test_aflex_update(self, mocked_get):
        mocked_get.side_effect = acos_errors.NotFound
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        filename = "testaflexpolicy"
        script = "when RULE_INIT{}"
        size = len(script.encode('utf-8'))
        action = "import"

        resp = self.client.slb.aflex_policy.update(filename, script, size, action)
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertIn(filename, responses.calls[1].request.body.decode('UTF-8'))
        self.assertIn(action, responses.calls[1].request.body.decode('UTF-8'))

    @mock.patch('acos_client.v30.slb.aflex_policy.AFlexPolicy.get')
    @responses.activate
    def test_aflex_delete(self, mocked_get):
        mocked_get.side_effect = acos_errors.NotFound
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        filename = "testaflexpolicy"

        resp = self.client.slb.aflex_policy.delete(filename)
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertIn(filename, responses.calls[1].request.body.decode('UTF-8'))
        self.assertIn('delete', responses.calls[1].request.body.decode('UTF-8'))



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import acos_client.errors as acos_errors
import responses


HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/services/rest/v2.1/?format=json&method='.format(HOSTNAME)
AUTH_URL = '{}authenticate'.format(BASE_URL)
CLOSE_AUTH_URL = '{}session.close&session_id={}'.format(BASE_URL, 'foobar')
BAD_CLOSE_AUTH_URL = '{}session.close&session_id={}'.format(BASE_URL, 'bad_foobar')
SYS_PARTITION_ACTIVE_URL = '{}system.partition.active&session_id={}'.format(BASE_URL, 'foobar')


class TestSession(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_session_id_is_none_at_init(self):
        json_response = {'session_id': 'foobar'}
        responses.add(responses.POST, AUTH_URL, json=json_response)

        resp = self.client.session.session_id

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 0)

    @responses.activate
    def test_session_id(self):
        json_response = {'session_id': 'foobar'}
        responses.add(responses.POST, AUTH_URL, json=json_response)

        resp = self.client.session.id

        self.assertEqual(resp, 'foobar')
        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(responses.calls[0].request.method, responses.POST)
        self.assertEqual(responses.calls[0].request.url, AUTH_URL)

    @responses.activate
    def test_session_authenticate_good_credentials(self):
        json_response = {'session_id': 'foobar'}
        responses.add(responses.POST, AUTH_URL, json=json_response)

        resp = self.client.session.authenticate('fake_username', 'fake_password')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(responses.calls[0].request.method, responses.POST)
        self.assertEqual(responses.calls[0].request.url, AUTH_URL)

    @responses.activate
    def test_session_authenticate_bad_credentials(self):
        json_response = {"response": {"status": "fail", "err": {"code": 520486915, "msg": " Admin password error"}}}
        responses.add(responses.POST, AUTH_URL, json=json_response)

        with self.assertRaises(acos_errors.AuthenticationFailure):
            self.client.session.authenticate('bad_username', 'bad_password')

        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(responses.calls[0].request.method, responses.POST)
        self.assertEqual(responses.calls[0].request.url, AUTH_URL)

    @responses.activate
    def test_session_close_good_session_id(self):
        auth_json_response = {'session_id': 'foobar'}
        responses.add(responses.POST, AUTH_URL, json=auth_json_response, status=200)
        close_json_response = {'response': {'status': 'OK'}}
        responses.add(responses.POST, CLOSE_AUTH_URL, json=close_json_response, status=200)

        self.client.session.id
        self.assertEqual(self.client.session.session_id, 'foobar')

        resp = self.client.session.close()

        self.assertEqual(resp, close_json_response)
        self.assertEqual(self.client.session.session_id, None)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CLOSE_AUTH_URL)

    @responses.activate
    def test_session_close_bad_session_id(self):
        auth_json_response = {'session_id': 'foobar'}
        responses.add(responses.POST, AUTH_URL, json=auth_json_response, status=200)
        close_json_response = {"response": {"status": "fail", "err": {"code": 1009, "msg": "Invalid session ID"}}}
        responses.add(responses.POST, BAD_CLOSE_AUTH_URL, json=close_json_response, status=200)

        self.client.session.id
        self.assertEqual(self.client.session.session_id, 'foobar')
        self.client.session.session_id = 'bad_foobar'

        resp = self.client.session.close()

        self.assertEqual(resp, close_json_response)
        self.assertEqual(self.client.session.session_id, None)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, BAD_CLOSE_AUTH_URL)



# Copyright 2016, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v30 import base


class Vlan(base.BaseV30):
    def __init__(self, client):
        super(Vlan, self).__init__(client)
        self.url_prefix = "/network/vlan"

    def _build_id_url(self, vlan_id):
        return "{0}/{1}".format(self.url_prefix, vlan_id)

    def get_list(self):
        return self._get(self.url_prefix)

    def create(self, vlan_id, name=None, shared_vlan=False,
               untagged_eths=[], untagged_trunks=[],
               tagged_eths=[], tagged_trunks=[], veth=False,
               lif=None):

        payload = {
            "vlan": self._build_payload(
                name, vlan_id, shared_vlan, untagged_eths,
                untagged_trunks,
                tagged_eths, tagged_trunks,
                veth, lif)
        }
        return self._post(self.url_prefix, payload)

    def get(self, vlan_id):
        return self._get(self._build_id_url(vlan_id))

    def exists(self, vlan_id):
        try:
            self.get(vlan_id)
            return True
        except acos_errors.NotFound:
            return False

    def delete(self, vlan_id):
        return self._delete(self._build_id_url(vlan_id))

    def _build_payload(self, name, vlan_id, shared_vlan,
                       untagged_eths, untagged_trunks,
                       tagged_eths, tagged_trunks,
                       veth, lif):
        rv = {
            "vlan-num": vlan_id,
        }

        if name:
            rv["name"] = name

        if shared_vlan is True:
            rv["shared-vlan"] = shared_vlan

        if untagged_eths:
            rv.update(self._build_range_list("untagged-eth", untagged_eths, "untagged-ethernet"))

        if tagged_eths:
            rv.update(self._build_range_list("tagged-eth", tagged_eths, "tagged-ethernet"))

        if untagged_trunks:
            rv.update(self._build_range_list("untagged-trunk", untagged_trunks))

        if tagged_trunks:
            rv.update(self._build_range_list("tagged-trunk", tagged_trunks))

        if veth:
            rv["ve"] = vlan_id

        if lif:
            rv["untagged-lif"] = lif

        return rv

    def _build_range_list(self, prefix="", xlist=[], inconsistent_prefix=""):
        # Naive way of building this poorly-conceived API call
        rv = []
        ekey_f = "{0}-list"
        rkey_f = "{0}-{1}"

        # Justification for snarky prefix
        fix_prefix = lambda: inconsistent_prefix if inconsistent_prefix else prefix

        for x in xlist:
            lm = {}
            lm[rkey_f.format(fix_prefix(), "start")] = x
            lm[rkey_f.format(fix_prefix(), "end")] = x
            rv.append(lm)

        return {ekey_f.format(prefix): rv}



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals
import six

from acos_client.v30 import base


class Member(base.BaseV30):

    url_base_tmpl = '/slb/service-group/{gname}/member/'
    url_mbr_tmpl = '{name}+{port}/'

    STATUS_ENABLE = 0
    STATUS_DISABLE = 1

    def get(self, service_group_name, server_name, server_port, **kwargs):
        url = self.url_base_tmpl.format(gname=service_group_name)
        url += self.url_mbr_tmpl.format(
            name=server_name,
            port=server_port
        )

        return self._get(url, **kwargs)

    def get_oper(self, service_group_name, server_name, server_port, **kwargs):
        url = self.url_base_tmpl.format(gname=service_group_name)
        url += self.url_mbr_tmpl.format(
            name=server_name,
            port=server_port
        )
        return self._get(url + 'oper', **kwargs)

    def _set(self,
             service_group_name,
             server_name,
             server_port,
             status=STATUS_ENABLE,
             member_state=True, **kwargs):
        params = {
            "member": self.minimal_dict({
                "name": server_name,
                "port": int(server_port),
                # flip status code, becuase it's a disable flag in v30
                "member-stats-data-disable": status,
                "member-state": member_state and 'enable' or 'disable',
            })
        }

        config_defaults = kwargs.get("config_defaults")

        if config_defaults:
            for k, v in six.iteritems(config_defaults):
                params['member'][k] = v

        return params

    def create(self,
               service_group_name,
               server_name,
               server_port,
               status=STATUS_ENABLE,
               member_state=True, **kwargs):
        url = self.url_base_tmpl.format(gname=service_group_name)
        params = self._set(service_group_name,
                           server_name, server_port, status, member_state, **kwargs)
        return self._post(url, params, **kwargs)

    def update(self,
               service_group_name,
               server_name,
               server_port,
               status=STATUS_ENABLE,
               member_state=True, **kwargs):
        url = self.url_base_tmpl.format(gname=service_group_name)
        url += self.url_mbr_tmpl.format(
            name=server_name,
            port=server_port
        )
        params = self._set(service_group_name,
                           server_name, server_port, status, member_state, **kwargs)
        return self._post(url, params, **kwargs)

    def replace(self,
                service_group_name,
                server_name,
                server_port,
                status=STATUS_ENABLE,
                member_state=True, **kwargs):
        url = self.url_base_tmpl.format(gname=service_group_name)
        url += self.url_mbr_tmpl.format(name=server_name,
                                        port=server_port)
        params = self._set(service_group_name,
                           server_name, server_port, status, member_state, **kwargs)
        return self._put(url, params, **kwargs)

    def delete(self, service_group_name, server_name, server_port):
        url = self.url_base_tmpl.format(gname=service_group_name)
        url += self.url_mbr_tmpl.format(
            name=server_name,
            port=server_port
        )
        self._delete(url)



# Copyright 2014-2016, A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
except ImportError:
    import unittest2 as unittest

from acos_client import client
import acos_client.errors as acos_errors
import json
import responses


HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/axapi/v3'.format(HOSTNAME)
AUTH_URL = '{}/auth'.format(BASE_URL)
SERVICE_GROUP_NAME = 'test1'
CREATE_URL = '{}/slb/service-group/'.format(BASE_URL)
OBJECT_URL = '{}/slb/service-group/{}'.format(BASE_URL, SERVICE_GROUP_NAME)


class TestVirtualServer(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '30', 'fake_username', 'fake_password')

    @responses.activate
    def test_server_group_create(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.create('test1')

        self.assertEqual(resp, json_response)
        # One responses call for auth and one responses call for post
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_group_create_with_templates(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        templates = {
            'template-server': 'template_sv',
            'template-port': 'template_port',
            'template-policy': 'template-pl'
        }
        resp = self.client.slb.service_group.create('test1', service_group_templates=templates)
        params = {
            'service-group':
            {
                'health-check-disable': 0,
                'lb-method': 'round-robin',
                'name': 'test1',
                'protocol': 'tcp',
                'stateless-auto-switch': 0,
                'template-server': 'template_sv',
                'template-port': 'template_port',
                'template-policy': 'template-pl'
            }
        }

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_server_group_create_with_partial_templates(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        templates = {
            'template-server': 'template_sv',
            'template-port': 'template_port',
        }
        resp = self.client.slb.service_group.create('test1', service_group_templates=templates)

        params = {
            'service-group':
            {
                'health-check-disable': 0,
                'lb-method': 'round-robin',
                'name': 'test1',
                'protocol': 'tcp',
                'stateless-auto-switch': 0,
                'template-server': 'template_sv',
                'template-port': 'template_port'
            }
        }

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_server_group_update(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.update('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_server_group_replace(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.PUT, OBJECT_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.replace('test1', protocl='tcp')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.PUT)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_server_group_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.DELETE, OBJECT_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.delete('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_server_group_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67239937, "msg": " No such Virtual Server"}}
        }
        responses.add(responses.DELETE, OBJECT_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.ACOSException):
            self.client.slb.service_group.delete('test1')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_server_group_stats(self):
        STATS_URL = '{}/slb/service-group/{}/stats'.format(BASE_URL, SERVICE_GROUP_NAME)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, STATS_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.stats('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, STATS_URL)

    @responses.activate
    def test_server_group_oper(self):
        OPER_URL = '{}/slb/service-group/{}/oper'.format(BASE_URL, SERVICE_GROUP_NAME)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, OPER_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.oper('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, OPER_URL)

    @responses.activate
    def test_server_group_all(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, CREATE_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.all('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_group_all_stats(self):
        ALL_STATS_URL = '{}/slb/service-group/stats'.format(BASE_URL)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, ALL_STATS_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.all_stats('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, ALL_STATS_URL)

    @responses.activate
    def test_server_group_all_oper(self):
        ALL_OPER_URL = '{}/slb/service-group/oper'.format(BASE_URL)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, ALL_OPER_URL, json=json_response, status=200)

        resp = self.client.slb.service_group.all_oper('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, ALL_OPER_URL)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import acos_client.errors as acos_errors
import responses


HOSTNAME = 'fake_a10'
BASE_URL = "https://{}:443/axapi/v3/".format(HOSTNAME)
AUTH_URL = "{}auth".format(BASE_URL)
SLB_URL = '{}slb/template/persist/cookie/'.format(BASE_URL)


class TestSLBTemplateCookiePersistence(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '30', 'fake_username', 'fake_password')

    @responses.activate
    def test_slb_template_persistence_create(self):
        # NOTE: Create method tries GET request to see if template exists before using POST request to create template.
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        get_url = '{}test1'.format(SLB_URL)
        get_json_response = {
            'response': {'status': 'fail', 'err': {
                'code': 1023460352, 'from': 'CM', 'msg': 'Object specified does not exist (object: cookie)'}}
        }
        responses.add(responses.GET, get_url, json=get_json_response, status=404)
        post_url = SLB_URL
        post_json_response = {
            'cookie': {'name': 'test1', 'dont-honor-conn-rules': 0, 'insert-always': 0, 'encrypt-level': 1,
                       'encrypted': 'foo', 'cookie-name': 'sto-id', 'path': '/', 'pass-thru': 0, 'secure': 0,
                       'httponly': 0, 'match-type': 0, 'uuid': '40143610-6a61-11e8-9bf6-c3c3b3edcd4e'}
        }
        responses.add(responses.POST, post_url, json=post_json_response, status=200)

        resp = self.client.slb.template.cookie_persistence.create('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 3)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, get_url)
        self.assertEqual(responses.calls[2].request.method, responses.POST)
        self.assertEqual(responses.calls[2].request.url, post_url)

    @responses.activate
    def test_slb_template_persistence_create_already_exists(self):
        # NOTE: Create method tries GET request to see if template exists before using POST request to create template.
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        url = '{}test1'.format(SLB_URL)
        json_response = {
            'response': {'status': 'fail', 'err': {
                'code': 1023459339, 'from': 'CM', 'msg': 'Failed to handle object "cookie". Object already exists',
                'location': 'cookie'}}
        }
        responses.add(responses.GET, url, json=json_response, status=200)

        with self.assertRaises(acos_errors.ACOSException):
            self.client.slb.template.cookie_persistence.create('test1')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, url)

    @responses.activate
    def test_slb_template_persistence_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        url = '{}test1'.format(SLB_URL)
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.DELETE, url, json=json_response, status=200)

        resp = self.client.slb.template.cookie_persistence.delete('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, url)

    @responses.activate
    def test_slb_template_persistence_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        url = '{}test1'.format(SLB_URL)
        json_response = {
            'response': {'status': 'fail', 'err': {
                'code': 1023460352, 'from': 'CM', 'msg': 'Object specified does not exist (object: cookie)'}}
        }
        responses.add(responses.DELETE, url, json=json_response, status=404)

        resp = self.client.slb.template.cookie_persistence.delete('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, url)

    @responses.activate
    def test_slb_template_persistence_search(self):
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        url = '{}test1'.format(SLB_URL)
        json_response = {
            'cookie-list': [{
                'name': 'test1', 'dont-honor-conn-rules': 0, 'insert-always': 0, 'encrypt-level': 1,
                'encrypted': 'foo', 'cookie-name': 'sto-id', 'path': '/', 'pass-thru': 0, 'secure': 0, 'httponly': 0,
                'match-type': 0, 'uuid': '40143610-6a61-11e8-9bf6-c3c3b3edcd4e',
                'a10-url': '/axapi/v3/slb/template/persist/cookie/test1'}]
        }
        responses.add(responses.GET, url, json=json_response, status=200)

        resp = self.client.slb.template.cookie_persistence.get('test1')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, url)

    @responses.activate
    def test_slb_template_persistence_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'authresponse': {'signature': 'foo', 'description': 'bar.'}})
        url = '{}test1'.format(SLB_URL)
        json_response = {
            'response': {'status': 'fail', 'err': {
                'code': 1023460352, 'from': 'CM', 'msg': 'Object specified does not exist (object: cookie)'}}
        }
        responses.add(responses.GET, url, json=json_response, status=404)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.template.cookie_persistence.get('test1')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, url)



# Copyright 2015, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v30 import base


class Nat(base.BaseV30):
    @property
    def pool(self):
        return self.Pool(self.client)

    class Pool(base.BaseV30):
        url_prefix = "/ip/nat/pool/"

        def _set(self, name, start_ip, end_ip, mask, ip_rr=None, vrid=None, **kwargs):
            params = {
                "pool": self.minimal_dict(
                    {
                        'pool-name': name,
                        'start-address': start_ip,
                        'end-address': end_ip,
                        'netmask': mask,
                    }
                ),
            }

            if ip_rr:
                params["pool"]["ip-rr"] = ip_rr

            if vrid:
                params["pool"]["vrid"] = vrid

            if self.exists(name):
                self._post(self.url_prefix + name, params, **kwargs)
            else:
                self._post(self.url_prefix, params, **kwargs)

        def get(self, name):
            return self._get(self.url_prefix + name)

        def exists(self, name):
            try:
                self.get(name)
                return True
            except acos_errors.NotFound:
                return False

        def all(self):
            return self._get(self.url_prefix)

        def create(self, name, start_ip, end_ip, mask, ip_rr, vrid, **kwargs):
            if self.exists(name):
                raise acos_errors.Exists

            self._set(name, start_ip, end_ip, mask, ip_rr, vrid, **kwargs)

        def delete(self, name, **kwargs):
            self._delete(self.url_prefix + name)

        def stats(self, name='', **kwargs):
            return self._get(self.url_prefix + name + '/stats', **kwargs)

        def all_stats(self, **kwargs):
            return self.stats()



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30.license_manager import LicenseManager

URL_BASE = "/axapi/v3/license-manager"


class TestLicenseManager(unittest.TestCase):
    def setUp(self):
        self.client = mock.Mock()
        self.target = LicenseManager(self.client)

    def _untested(self):
        raise NotImplementedError("This needs actual test code")

    def _test_create(self, hosts, serial, payload):
        self.target.create(hosts, serial)
        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertEqual("POST", method)
        self.assertEqual(URL_BASE, url)
        self.assertEqual(payload, params)
        return {"method": method, "url": url, "params": params, "header": header, "kwargs": kwargs}

    def test_create(self):
        hosts = [{"ip": "127.0.0.1", "port": 443}]
        serial = "sn1234567890"

        payload = self.target._build_payload(hosts, serial)

        self._test_create(hosts, serial, payload)

    def test_create_multihosts(self):
        hosts = [{"ip": "127.0.0.1", "port": 443},
                 {"ip": "127.0.0.2", "port": 443}]
        serial = "sn1234567890"

        payload = self.target._build_payload(hosts, serial)

        self._test_create(hosts, serial, payload)

    def test_create_use_mgmt_port(self):
        hosts = [{"ip": "127.0.0.1", "port": 443}]
        serial = "sn1234567890"
        expected = False
        payload = self.target._build_payload(hosts, serial, use_mgmt_port=expected)

        result = self._test_create(hosts, serial, payload)

        actual = result["params"]["license-manager"]["use-mgmt-port"]
        self.assertEqual(expected, actual)

    def test_create_serial(self):
        hosts = [{"ip": "127.0.0.1", "port": 443}]
        serial = "sn1234567890"

        payload = self.target._build_payload(hosts, serial)

        result = self._test_create(hosts, serial, payload)

        actual = result["params"]["license-manager"]["sn"]
        self.assertEqual(serial, actual)

    def test_get(self):
        self.target.get()
        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertEqual("GET", method)
        self.assertEqual(URL_BASE, url)

    def test_connect_positive(self):
        self._test_connect(True)

    def test_connect_false(self):
        self._test_connect(False)

    def _test_connect(self, connect):
        self.target.connect(connect)

        payload = {
            "connect": {
                "connect": 1 if connect else 0
            }
        }

        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertEqual("POST", method)
        self.assertEqual(URL_BASE + "/connect", url)
        self.assertEqual(params, payload)

    def test_update(self):
        hosts = [{"ip": "127.0.0.2", "port": 443}]
        serial = "sn1234567890"

        payload = self.target._build_payload(hosts, serial)
        self.target.create(hosts, serial)
        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertEqual("POST", method)
        self.assertEqual(URL_BASE, url)
        self.assertEqual(payload, params)
        return {"method": method, "url": url, "params": params, "header": header, "kwargs": kwargs}



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30 import dns


class TestDns(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = dns.DNS(self.client)
        self.url_prefix = "/axapi/v3/ip/dns/"

    def test_primary_ipv4(self):
        expected = '192.0.2.4'
        self.target.set(primary=expected)

        expected_payload = {'primary': {'ip-v4-addr': expected}}

        self.client.http.request.assert_called_with("POST", self.url_prefix + 'primary',
                                                    expected_payload, mock.ANY)

    def test_primary_ipv6(self):
        expected = '0:0:0:0:0:FFFF:129.144.52.38'
        self.target.set(primary=expected)

        expected_payload = {'primary': {'ip-v6-addr': expected}}

        self.client.http.request.assert_called_with("POST", self.url_prefix + 'primary',
                                                    expected_payload, mock.ANY)

    def test_secondary_ipv4(self):
        expected = '192.0.2.5'
        self.target.set(secondary=expected)

        expected_payload = {'secondary': {'ip-v4-addr': expected}}

        self.client.http.request.assert_called_with("POST", self.url_prefix + 'secondary',
                                                    expected_payload, mock.ANY)

    def test_secondary_ipv6(self):
        expected = '0:0:0:0:0:FFFF:129.144.52.39'
        self.target.set(secondary=expected)

        expected_payload = {'secondary': {'ip-v6-addr': expected}}

        self.client.http.request.assert_called_with("POST", self.url_prefix + 'secondary',
                                                    expected_payload, mock.ANY)

    def test_suffix(self):
        expected = 'example.com'
        self.target.set(suffix=expected)

        expected_payload = {'suffix': {'domain-name': expected}}

        self.client.http.request.assert_called_with("POST", self.url_prefix + 'suffix',
                                                    expected_payload, mock.ANY)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class Network(base.BaseV21):

    @property
    def interface(self):
        return self.Interface(self.client)

    class Interface(base.BaseV21):

        def all(self):
            return self._get('network.interface.getAll')

        def get(self, port_num):
            return self._get('network.interface.get', {"port_num": port_num})

        def set(self, port_num, **kwargs):

            params = {
                "interface": {
                    "port_num": port_num
                }
            }
            return self._post('network.interface.set', params, **kwargs)

        @property
        def ipv4(self):
            return self.IPV4(self.client)

        class IPV4(base.BaseV21):

            def _set(self, action, port_num, ipv4_address, ipv4_mask,
                     **kwargs):

                params = {
                    "interface": {
                        "ipv4": {
                            "ipv4_address": ipv4_address,
                            "ipv4_mask": ipv4_mask
                        },
                        "port_num": port_num
                    }
                }

                return self._post(action, params, **kwargs)

            def all_delete(self, port_num, **kwargs):
                return self._post('network.interface.ipv4.deleteAll',
                                  {"port_num": port_num}, **kwargs)

            def add(self, port_num, ipv4_address, ipv4_mask, **kwargs):
                return self._set('network.interface.ipv4.add', port_num,
                                 ipv4_address, ipv4_mask, **kwargs)

            def delete(self, port_num, ipv4_address, ipv4_mask, **kwargs):
                return self._set('network.interface.ipv4.delete', port_num,
                                 ipv4_address, ipv4_mask, **kwargs)

    @property
    def acl(self):
        return self.ACL(self.client)

    class ACL(base.BaseV21):

        @property
        def ext(self):
            return self.Ext(self.client)

        class Ext(base.BaseV21):

            # Protocols
            ICMP = 0
            IP = 1
            TCP = 2
            UDP = 3

            def _set(self, action, id, acl_item_list, **kwargs):

                params = {
                    "ext_acl": {
                        "id": id,
                        'acl_item_list': acl_item_list
                    }
                }

                # silently fails if 'acl_item_list' is empty
                return self._post(action, params, **kwargs)

            def all(self):
                return self._get('network.acl.ext.getAll')

            def search(self, id):
                # FIXME not working
                return self._get('network.acl.ext.search', {"id": id})

            def create(self, id, acl_item_list, **kwargs):
                return self._set('network.acl.ext.create', id, acl_item_list,
                                 **kwargs)

            def update(self, id, acl_item_list, **kwargs):
                return self._set('network.acl.ext.update', id, acl_item_list,
                                 **kwargs)  # FIXME NOT WORKING

            def delete(self, id):
                # FIXME NOT WORKING
                return self._post('network.acl.ext.delete', {"id": id})

            def all_delete(self, **kwargs):
                return self._get('network.acl.ext.deleteAll', **kwargs)

    @property
    def route(self):
        return self.Route(self.client)

    class Route(base.BaseV21):

        def _set(self, action, address, mask, gateway, distance, **kwargs):
            params = {
                'address': address,
                'mask': mask,
                'gateway': gateway,
                'distance': distance
            }
            return self._post(action, params, **kwargs)

        def ipv4_all(self, **kwargs):
            return self._get('network.route.ipv4static.getAll', **kwargs)

        def ipv4_create(self, address, mask, gateway, distance, **kwargs):
            return self._set('network.route.ipv4static.create', address, mask,
                             gateway, distance, **kwargs)

        def ipv4_update(self, address, mask, gateway, distance, **kwargs):
            return self._set('network.route.ipv4static.update', address, mask,
                             gateway, distance, **kwargs)  # FIXME NOT WORKING

        def ipv4_delete(self, address, mask, gateway, distance, **kwargs):
            return self._set('network.route.ipv4static.delete', address, mask,
                             gateway, distance, **kwargs)  # FIXME NOT WORKING



# Copyright 2015,  Tobit Raff,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals
import six


from acos_client import errors as acos_errors
from acos_client.v30 import base


class SSLKey(base.BaseV30):

    url_prefix = '/file/ssl-key/'

    def get(self, file):
        return self._get(self.url_prefix + file)

    def exists(self, file):
        try:
            self.get(file)
            return True
        except acos_errors.NotFound:
            return False

    def _set(self, file="", cert="", size="", action="", **kwargs):

        obj_params = {
            "file": file,
            "size": size,
            "file-handle": file,
            "action": action,
        }

        kwargs['params'] = {'ssl-key': {}}

        for key, val in six.iteritems(obj_params):
            # Filter out invalid, or unset keys
            if val != "":
                kwargs['params']['ssl-key'][key] = val

        return self._post(self.url_prefix, file_name=obj_params["file"],
                          file_content=cert, **kwargs)

    def create(self, file="", cert="", size="", action=""):
        if self.exists(file):
            raise acos_errors.Exists

        self._set(file, cert, size, action)

    def update(self, file="", cert="", size="", action=""):
        self._set(file, cert, size, action)

    def delete(self, private_key=""):
        payload = {"delete": {"private-key": private_key}}
        self._request("POST", "/pki/delete", payload)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base

from acos_client.v21.slb.aflex import Aflex
from acos_client.v21.slb.class_list import ClassList
from acos_client.v21.slb.hm import HealthMonitor
from acos_client.v21.slb.server import Server
from acos_client.v21.slb.service_group import ServiceGroup
from acos_client.v21.slb.template import Template
from acos_client.v21.slb.virtual_server import VirtualServer
from acos_client.v21.slb.virtual_service import VirtualService


class SLB(base.BaseV21):
    # For status args
    DOWN = 0
    UP = 1

    @property
    def hm(self):
        return HealthMonitor(self.client)

    @property
    def server(self):
        return Server(self.client)

    @property
    def service_group(self):
        return ServiceGroup(self.client)

    @property
    def template(self):
        return Template(self.client)

    @property
    def virtual_server(self):
        return VirtualServer(self.client)

    @property
    def aflex(self):
        return Aflex(self.client)

    @property
    def class_list(self):
        return ClassList(self.client)

    @property
    def virtual_service(self):
        return VirtualService(self.client)

    @property
    def common(self):
        raise NotImplementedError("slb.common support is not available using AXAPI v2.1")


class File(base.BaseV21):
    @property
    def ssl_key(self):
        raise NotImplementedError("Not implemented in AXAPI v2.1")

    @property
    def ssl_cert(self):
        raise NotImplementedError("Not implemented in AXAPI v2.1")



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

import time

from acos_client import errors as acos_errors


class BaseV21(object):

    def __init__(self, client):
        self.client = client

    def minimal_dict(self, my_dict):
        return dict((k, v) for k, v in my_dict.items() if v is not None)

    def url(self, action):
        return ("/services/rest/v2.1/?format=json&method=%s&session_id=%s" %
                (action, self.client.session.id))

    def _request(self, method, action, params, retry_count=0, **kwargs):
        if retry_count > 6:
            raise acos_errors.ACOSUnknownError()

        try:
            return self.client.http.request(method, self.url(action), params,
                                            **kwargs)
        except acos_errors.MemoryFault as e:
            if retry_count < 5:
                time.sleep(0.1)
                return self._request(method, action, params, retry_count + 1, **kwargs)
            raise e
        except acos_errors.InvalidSessionID as e:
            if retry_count < 5:
                time.sleep(0.1)
                try:
                    p = self.client.current_partition
                    self.client.session.close()
                    self.client.partition.active(p)
                except Exception:
                    pass
                return self._request(method, action, params, retry_count + 1, **kwargs)
            raise e

    def _get(self, action, params={}, **kwargs):
        return self._request('GET', action, params, **kwargs)

    def _post(self, action, params={}, **kwargs):
        return self._request('POST', action, params, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import acos_client.errors as acos_errors
import responses


HOSTNAME = 'fake_a10'
BASE_URL = "https://{}:443/services/rest/v2.1/?format=json&method=".format(HOSTNAME)
AUTH_URL = "{}authenticate".format(BASE_URL)
AUTH_CLOSE_URL = "{}session.close&session_id=None".format(BASE_URL)
CREATE_URL = '{}system.partition.create&session_id={}'.format(BASE_URL, 'foobar')
DELETE_URL = '{}system.partition.delete&session_id={}'.format(BASE_URL, 'foobar')
SEARCH_URL = '{}system.partition.search&session_id={}'.format(BASE_URL, 'foobar')
ACTIVE_URL = '{}system.partition.active&session_id={}'.format(BASE_URL, 'foobar')


class TestPartition(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_system_partition_create(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        resp = self.client.system.partition.create('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_system_partition_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1982, "msg": "The partition already exists"}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.Exists):
            self.client.system.partition.create('test1')
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_system_partition_delete(self):
        responses.add(responses.POST, AUTH_CLOSE_URL)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)
        resp = self.client.system.partition.delete('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 3)
        self.assertEqual(responses.calls[2].request.method, responses.POST)
        self.assertEqual(responses.calls[2].request.url, DELETE_URL)

    @responses.activate
    def test_system_partition_delete_not_found(self):
        responses.add(responses.POST, AUTH_CLOSE_URL)
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 520749062, "msg": " Partition does not exist."}}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.system.partition.delete('test1')
        self.assertEqual(len(responses.calls), 3)
        self.assertEqual(responses.calls[2].request.method, responses.POST)
        self.assertEqual(responses.calls[2].request.url, DELETE_URL)

    @responses.activate
    def test_system_partition_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "partition": {"partition_id": 1, "name": "p1", "max_aflex_file": 32, "network_partition": 0}
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.system.partition.exists('test1')

        self.assertEqual(resp, True)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)

    @responses.activate
    def test_system_partition_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 520749062, "msg": " Partition does not exist."}}
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.system.partition.exists('test1')

        self.assertEqual(resp, False)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)

    @responses.activate
    def test_system_partition_active(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, ACTIVE_URL, json=json_response, status=200)

        resp = self.client.system.partition.active('test1')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, ACTIVE_URL)

    @responses.activate
    def test_system_partition_active_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 402718800, "msg": " Failed to get partition."}}
        }
        responses.add(responses.POST, ACTIVE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.system.partition.active('test1')
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, ACTIVE_URL)



# Copyright 2014-2016,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30.slb import common


class TestSFlow(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = common.SLBCommon(self.client)

    def test_underscore_to_dash(self):
        expected = "no-dsr-health-check"
        actual = self.target._underscore_to_dash(expected)
        self.assertEqual(expected.replace('_', '-'), actual)

    def test_create(self):
        in_params = {"no_dsr_health_check": 1, "nounderscores": "string", "boolean": False}
        expected_params = {"no-dsr-health-check": 1, "nounderscores": "string", "boolean": False}
        self.target.create(**in_params)
        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertTrue('GET', method)
        self.assertTrue('/axapi/v3/' + self.target.url_prefix, url)
        self.assertTrue(expected_params, params)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

import logging
import six
import socket

import acos_client
from acos_client import errors as acos_errors
from acos_client.v21 import axapi_http as v21_http
from acos_client.v21.dns import DNS as v21_DNS
from acos_client.v21.ha import HA as v21_HA
from acos_client.v21.interface import Interface as v21_Interface
from acos_client.v21.license_manager import LicenseManager as v21_LicenseManager
from acos_client.v21.nat import Nat as v21_Nat
from acos_client.v21.network import Network as v21_Network
from acos_client.v21.session import Session as v21_Session
from acos_client.v21.sflow import SFlow as v21_SFlow
from acos_client.v21.slb import SLB as v21_SLB
from acos_client.v21.system import System as v21_System
from acos_client.v21.vrrp_a import VRRPA as v21_VRRPA

from acos_client.v30 import axapi_http as v30_http
from acos_client.v30.device_context import DeviceContext as v30_DeviceContext
from acos_client.v30.dns import DNS as v30_DNS
from acos_client.v30.file import File as v30_File
from acos_client.v30.ha import HA as v30_HA
from acos_client.v30.interface import Interface as v30_Interface
from acos_client.v30.license_manager import LicenseManager as v30_LicenseManager
from acos_client.v30.nat import Nat as v30_Nat
from acos_client.v30.network import Network as v30_Network
from acos_client.v30.overlay import Overlay as v30_Overlay
from acos_client.v30.route import RIB as v30_RIB
from acos_client.v30.session import Session as v30_Session
from acos_client.v30.sflow import SFlow as v30_SFlow
from acos_client.v30.slb import SLB as v30_SLB
from acos_client.v30.system import System as v30_System
from acos_client.v30.vlan import Vlan as v30_Vlan
from acos_client.v30.vrrpa.vrid import VRID as v30_VRRPA

VERSION_IMPORTS = {
    '21': {
        'DNS': v21_DNS,
        'http': v21_http,
        'HA': v21_HA,
        'Interface': v21_Interface,
        'LicenseManager': v21_LicenseManager,
        'Nat': v21_Nat,
        'Network': v21_Network,
        'Session': v21_Session,
        'SFlow': v21_SFlow,
        'SLB': v21_SLB,
        'System': v21_System,
        'Vlan': None,
        'VRRPA': v21_VRRPA
    },
    '30': {
        'DNS': v30_DNS,
        'http': v30_http,
        'Interface': v30_Interface,
        'HA': v30_HA,
        'LicenseManager': v30_LicenseManager,
        'Nat': v30_Nat,
        'Network': v30_Network,
        'Overlay': v30_Overlay,
        'RIB': v30_RIB,
        'Session': v30_Session,
        'SFlow': v30_SFlow,
        'SLB': v30_SLB,
        'System': v30_System,
        'File': v30_File,
        'Vlan': v30_Vlan,
        'VRRPA': v30_VRRPA,
        'DeviceContext': v30_DeviceContext
    },
}

LOG = logging.getLogger(__name__)


class Client(object):

    def __init__(
            self,
            host,              # ip address or name of the A10 device
            version,           # either 21 or 30
            username,          # username to use for authenticating to the A10 device
            password,          # password to use for authenticating to the A10 device
            max_retries=3,     # number of times to retry a connection before giving up
            port=None,         # TCP port to use for connecting to the A10 device
            protocol="https",  # transport protocol - http or https, encryption recommended
            timeout=5          # seconds to wait for return data before giving up
    ):
        self._version = self._just_digits(version)
        if self._version not in acos_client.AXAPI_VERSIONS:
            raise acos_errors.ACOSUnsupportedVersion()
        self.max_retries = max_retries
        self.timeout = timeout
        self.host = host
        self.port = port
        self.http = VERSION_IMPORTS[self._version]['http'].HttpClient(
            host, port, protocol, max_retries=self.max_retries, timeout=timeout
        )
        self.session = VERSION_IMPORTS[self._version]['Session'](self, username, password)
        self.current_partition = 'shared'

    def _just_digits(self, s):
        return ''.join(i for i in str(s) if i.isdigit())

    @property
    def dns(self):
        return VERSION_IMPORTS[self._version]['DNS'](self)

    @property
    def ha(self):
        return VERSION_IMPORTS[self._version]['HA'](self)

    @property
    def interface(self):
        return VERSION_IMPORTS[self._version]['Interface'](self)

    @property
    def system(self):
        return VERSION_IMPORTS[self._version]['System'](self)

    @property
    def slb(self):
        return VERSION_IMPORTS[self._version]['SLB'](self)

    @property
    def network(self):
        return VERSION_IMPORTS[self._version]['Network'](self)

    @property
    def nat(self):
        return VERSION_IMPORTS[self._version]['Nat'](self)

    @property
    def file(self):
        return VERSION_IMPORTS[self._version]['File'](self)

    @property
    def sflow(self):
        return VERSION_IMPORTS[self._version]['SFlow'](self)

    @property
    def license_manager(self):
        return VERSION_IMPORTS[self._version]["LicenseManager"](self)

    @property
    def overlay(self):
        return VERSION_IMPORTS[self._version]["Overlay"](self)

    @property
    def vlan(self):
        return VERSION_IMPORTS[self._version]["Vlan"](self)

    @property
    def route(self):
        return VERSION_IMPORTS[self._version]["RIB"](self)

    @property
    def vrrpa(self):
        return VERSION_IMPORTS[self._version]["VRRPA"](self)

    @property
    def device_context(self):
        return VERSION_IMPORTS[self._version]["DeviceContext"](self)

    def wait_for_connect(self, max_timeout=60):
        for i in six.moves.range(0, max_timeout):
            try:
                LOG.debug("wait_for_connect: attempting %s", self.host)
                s = socket.create_connection((self.host, self.port), 1.0)
                s.close()
                LOG.debug("wait_for_connect: connected %s", self.host)
                break
            except socket.error:
                pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30.slb import port


class TestPort(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.port = port.Port(self.client)

        # common test parameter(s) throughout all test-cases
        self._server_name = 'test_server'

    def test_create_port(self):
        expected = {
            'port': {
                "conn-resume": None,
                "conn-limit": None,
                "stats-data-action": "stats-data-enable",
                "weight": 1,
                "port-number": 80,
                "range": 0,
                "action": "enable",
                "protocol": 'tcp'
            }
        }
        self.port.create('test_server', 80, 'tcp')

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/axapi/v3/slb/server/%s/port/' % self._server_name)
        self.assertEqual(params, expected)

    def test_update_port(self):
        expected = {
            'port': {
                "conn-resume": None,
                "conn-limit": 12345,
                "stats-data-action": "stats-data-enable",
                "weight": 2,
                "port-number": 80,
                "range": 0,
                "action": "enable",
                "protocol": 'tcp'
            }
        }
        self.port.update('test_server', 80, 'tcp', conn_limit=12345, weight=2)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(method, 'PUT')
        self.assertEqual(url, '/axapi/v3/slb/server/%s/port/%s+%s/' %
                              (self._server_name, 80, 'tcp'))
        self.assertEqual(params, expected)

    def test_delete_port(self):
        self.port.delete('test_server', 80, 'tcp')

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(method, 'DELETE')
        self.assertEqual(url, '/axapi/v3/slb/server/%s/port/%s+%s/' %
                              (self._server_name, 80, 'tcp'))



# Copyright 2016, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client.v30 import base


class BladeParameters(base.BaseV30):
    def __init__(self, client):
        super(BladeParameters, self).__init__(client)
        self.base_url = "/vrrp-a/vrid/{0}/blade-parameters"
        self.interfaces = {'interface': []}
        self.gateways = {
            'gateway': {
                'ipv4-gateway-list': [],
                'ipv6-gateway-list': []
            }
        }

    def _build_params(self, priority=None, **kwargs):
        rv = {'blade-parameters': {}}
        if priority:
            priority = priority if priority in range(1, 256) else 150
            rv['blade-parameters']['priority'] = priority

        if self.interfaces['interface']:
            rv['blade-parameters']['tracking-options'] = self.interfaces

        if self.gateways['gateway']['ipv4-gateway-list']:
            if rv['blade-parameters'].get('tracking-options'):
                rv['blade-parameters']['tracking-options'].update(self.gateways)
            else:
                rv['blade-parameters']['tracking-options'] = self.gateways

        if self.gateways['gateway']['ipv6-gateway-list']:
            if rv['blade-parameters'].get('tracking-options'):
                if rv['blade-parameters']['tracking-options'].get('gateway'):
                    rv['blade-parameters']['tracking-options']['gateway'].update(self.gateways)
                else:
                    rv['blade-parameters']['tracking-options'] = self.gateways
            else:
                rv['blade-parameters']['tracking-options'] = self.gateways
        return rv

    def add_interface(self, ethernet=1, priority_cost=1):
        interface = {
            'ethernet': ethernet,
            'priority-cost': priority_cost
        }
        self.interfaces['interface'].append(interface)

    def add_ipv4gateway(self, ip_address, priority_cost=1):
        gateway = {
            'ip-address': ip_address,
            'priority-cost': priority_cost
        }
        self.gateways['gateway']['ipv4-gateway-list'].append(gateway)

    def add_ipv6gateway(self, ip_address, priority_cost=1):
        gateway = {
            'ip-address': ip_address,
            'priority-cost': priority_cost
        }
        self.gateways['gateway']['ipv6-gateway-list'].append(gateway)

    def get(self, vrid_val):
        return self._get(self.base_url.format(vrid_val))

    def create(self, vrid_val, priority=None, **kwargs):
        payload = self._build_params(priority, **kwargs)
        self._post(self.base_url.format(vrid_val), payload)

    def update(self, vrid_val, priority=None, **kwargs):
        payload = self._build_params(priority, **kwargs)
        self._put(self.base_url.format(vrid_val), payload)

    def delete(self, vrid_val):
        return self._delete(self.base_url.format(vrid_val))



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class VRRPAGlobal(base.BaseV21):
    def get(self, **kwargs):
        return self._get("vrrp_a.get", **kwargs)

    def set(self, status, device_id, set_id, default_vrid, hello_interval, dead_timer,
            track_event_delay, preemption_delay, arp_retry,
            vrid_list={},
            preferred_session_sync_port_list={}):
        params = {
            "vrrp_a": {
                "status": status,
                "device_id": device_id,
                "set_id": set_id,
                "default_vrid": default_vrid,
                "hello_interval": hello_interval,
                "dead_timer": dead_timer,
                "track_event_delay": track_event_delay,
                "preemption_delay": preemption_delay,
                "arp_retry": arp_retry,
            }
        }
        vrids = self._convert_vrid_list(vrid_list)
        sync_ports = self._convert_sync_port_list(preferred_session_sync_port_list)

        if vrids:
            params["vrrp_a"]["vrid_list"] = vrids

        if sync_ports:
            params["vrrp_a"]["preferred_session_sync_port_list"] = sync_ports

        return self._post("vrrp_a.set", params)

    def _convert_vrid_list(self, vrids):
        return vrids

    def _convert_sync_port_list(self, sync_ports):
        return sync_ports



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import acos_client.errors as acos_errors
import json
import responses

HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/services/rest/v2.1/?format=json&method='.format(HOSTNAME)
AUTH_URL = '{}authenticate'.format(BASE_URL)
SESSION_ID = 'foobar'
CREATE_URL = '{}slb.virtual_server.create&session_id={}'.format(BASE_URL, SESSION_ID)
UPDATE_URL = '{}slb.virtual_server.update&session_id={}'.format(BASE_URL, SESSION_ID)
DELETE_URL = '{}slb.virtual_server.delete&session_id={}'.format(BASE_URL, SESSION_ID)
SEARCH_URL = '{}slb.virtual_server.search&session_id={}'.format(BASE_URL, SESSION_ID)
STATS_URL = '{}slb.virtual_server.fetchStatistics&session_id={}'.format(BASE_URL, SESSION_ID)
ALL_STATS_URL = '{}slb.virtual_server.fetchAllStatistics&session_id={}'.format(BASE_URL, SESSION_ID)


class TestVirtualServer(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_virtual_server_create_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '192.168.2.254',
                'name': 'test',
                'status': 1
            }
        }

        resp = self.client.slb.virtual_server.create('test', '192.168.2.254')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_create_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '192.168.2.254',
                'name': 'test',
                'status': 1,
                'vrid': 1,
                'vip_template': 'TEST_VIP_TEMPLATE',

            }
        }

        resp = self.client.slb.virtual_server.create(
            name='test',
            ip_address='192.168.2.254',
            status=1,
            vrid=1,
            template_virtual_server='TEST_VIP_TEMPLATE',
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 402653206, "msg": " Name already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.Exists):
            self.client.slb.virtual_server.create('test', '192.168.2.254')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_virtual_server_update_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '192.168.2.254',
                'name': 'test',
                'status': 1
            }
        }

        resp = self.client.slb.virtual_server.update('test', '192.168.2.254')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_update_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '192.168.2.254',
                'name': 'test',
                'status': 1,
                'vrid': 1,
                'vip_template': 'TEST_VIP_TEMPLATE',

            }
        }

        resp = self.client.slb.virtual_server.update(
            name='test',
            ip_address='192.168.2.254',
            status=1,
            vrid=1,
            template_virtual_server='TEST_VIP_TEMPLATE',
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.delete('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.delete('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.get('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67239937, "msg": " No such Virtual Server"}}
        }
        params = {'name': 'test'}
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.virtual_server.get('test')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_stats(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, STATS_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.stats('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, STATS_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_all_stats(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, ALL_STATS_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.all_stats()

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)


class TestIPv6VirtualServer(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_virtual_server_create_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '2001:dbef:1111:feed:beef:8000:1d01:200f',
                'name': 'test',
                'status': 1
            }
        }

        resp = self.client.slb.virtual_server.create('test', '2001:dbef:1111:feed:beef:8000:1d01:200f')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_create_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '2001:dbef:1111:feed:beef:8000:1d01:200f',
                'name': 'test',
                'status': 1,
                'vrid': 1,
                'vip_template': 'TEST_VIP_TEMPLATE',

            }
        }

        resp = self.client.slb.virtual_server.create(
            name='test',
            ip_address='2001:dbef:1111:feed:beef:8000:1d01:200f',
            status=1,
            vrid=1,
            template_virtual_server='TEST_VIP_TEMPLATE',
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 402653206, "msg": " Name already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.Exists):
            self.client.slb.virtual_server.create('test', '2001:dbef:1111:feed:beef:8000:1d01:200f')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_virtual_server_update_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '2001:dbef:1111:feed:beef:8000:1d01:200f',
                'name': 'test',
                'status': 1
            }
        }

        resp = self.client.slb.virtual_server.update('test', '2001:dbef:1111:feed:beef:8000:1d01:200f')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_update_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            'virtual_server': {
                'address': '2001:dbef:1111:feed:beef:8000:1d01:200f',
                'name': 'test',
                'status': 1,
                'vrid': 1,
                'vip_template': 'TEST_VIP_TEMPLATE',

            }
        }

        resp = self.client.slb.virtual_server.update(
            name='test',
            ip_address='2001:dbef:1111:feed:beef:8000:1d01:200f',
            status=1,
            vrid=1,
            template_virtual_server='TEST_VIP_TEMPLATE',
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.delete('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.delete('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.get('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67239937, "msg": " No such Virtual Server"}}
        }
        params = {'name': 'test'}
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.virtual_server.get('test')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_stats(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        params = {'name': 'test'}
        responses.add(responses.POST, STATS_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.stats('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, STATS_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_server_all_stats(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, ALL_STATS_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.all_stats()

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, ALL_STATS_URL)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

import six
import time

from acos_client import errors as acos_errors
from acos_client.v21 import base


class Action(base.BaseV21):

    def write_memory(self):
        try:
            self._get("system.action.write_memory")
        except acos_errors.InvalidPartitionParameter:
            pass

    def reboot(self, **kwargs):
        raise NotImplementedError
        # return self._post("system.action.reboot", **kwargs)

    def reload(self, write_memory=False, **kwargs):
        # write_memory param is required but no matter what value is passed
        # it will ALWAYS save pending changes
        write_memory = 1 if write_memory else 0
        return self._post("system.action.reload",
                          params={"write_memory": write_memory}, **kwargs)

    def activate_and_write(self, partition, **kwargs):
        write_cmd = "write memory\r\n"

        if partition is not None:
            write_cmd = "active-partition {0}\r\n{1}".format(partition, write_cmd)

        last_e = None
        for i in six.moves.range(0, 5):
            # Request raises an exception when the "maybe error" is returned.
            try:
                return self._request("POST", "cli.deploy", params=None, payload=write_cmd, **kwargs)
            except acos_errors.ACOSException as e:
                last_e = e
                # Catch 'might fail error'
                if e.msg.startswith("write memory") or '2039 ' in e.msg:
                    time.sleep(1)
                    continue
                raise e

        if last_e is not None:
            raise last_e



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class OverlayVtep(base.BaseV30):
    url_prefix = "/overlay-tunnel/vtep"
    ip_url_format = "{baseurl}/{vtep}/{addrtype}"

    def get(self, vtep_id, *args, **kwargs):
        url = "{0}/{1}".format(self.url_prefix, vtep_id)
        return self._get(url, **kwargs)

    def get_list(self, *args, **kwargs):
        return self._get(self.url_prefix)

    def create(self, vtep_id, source_ip=None, source_vnis=[],
               dest_ips=[], lif_id=None, encap_type="vxlan", **kwargs):
        # vtep_id = ID
        # source_ip = source-ip-address
        # source_vnis = vni-list for source-ip-address
        # dest_ips = list of (ip, [vni_info]) tuples
        # vtep creation can be as simple or complicated as we like
        # we can create a vtep with a minimum of an ID
        # or we can create it with fully populated src/dst info

        is_found = False

        try:
            existing = self.get(vtep_id)
            is_found = existing is not None
        except Exception:
            pass

        if not is_found:
            # Create it.
            payload = {
                "vtep": {
                    "id": vtep_id,
                }
            }
            existing = self._post(self.url_prefix, payload, **kwargs)
        # iterate later.
        dest_ip = dest_ips[0]
        vni = source_vnis[0]
        if source_ip:
            payload, url = self._build_ip_payload_and_url(vtep_id, "source", source_ip, encap_type, vni)
            self._post(url, payload)

        if dest_ip:
            payload, url = self._build_ip_payload_and_url(vtep_id, "destination", dest_ip, encap_type, vni)
            self._post(url, payload)

        return self.get(vtep_id)

    def update(self, vtep_id, ip_address, encap_type="vxlan", **kwargs):
        post_url = "{0}/{vtepid}/destination-ip-address/".format(self.url_prefix, vtepid=vtep_id)
        payload = {
            "destination-ip-address": {
                "ip-address": ip_address,
            }
        }
        return self._post(post_url, payload, **kwargs)

    def delete(self, vtep_id, **kwargs):
        post_url = "{0}/{vtepid}".format(self.url_prefix, vtepid=vtep_id)

        return self._delete(post_url)

    def _add_source_address(self, vtep_id, ip_address, vnis=[]):
        pass

    def _add_destination_address(self, vtep_id, ip_address, vnis=[]):
        pass

    def _add_source_vni(self, vtep_id, ip_address, segment, partition=None, gateway=None, lif=None):
        pass

    def _add_destination_vni(self, vtep_id, ip_address, segment):
        pass

    def _build_ip_payload_and_url(self, vtep_id, target, ip_address, encap_type, vni, lif_id=None):
        addr_type = "{0}-ip-address".format(target)

        payload = {
            addr_type: {
                "ip-address": ip_address,
            }
        }
        if lif_id and target == "source":
            payload[addr_type]["vni-list"] = [{"segment": vni, "partition": "shared", "lif": lif_id}]

        if target == "destination":
            payload[addr_type]["encap"] = encap_type

        url = self.ip_url_format.format(baseurl=self.url_prefix, vtep=vtep_id, addrtype=addr_type)

        return payload, url



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals
import six


from acos_client import errors as acos_errors
from acos_client.v30 import base


class HealthMonitor(base.BaseV30):

    # Valid method objects
    UDP = 'udp'
    ICMP = 'icmp'
    TCP = 'tcp'
    HTTP = 'http'
    HTTPS = 'https'
    url_prefix = "/health/monitor/"

    _method_objects = {
        ICMP: {
            "icmp": 1
        },
        UDP: {
            "udp": 1,
            "udp-port": 5550,
            "force-up-with-single-healthcheck": 0
        },
        HTTP: {
            "http": 1,
            "http-port": 80,
            "http-expect": 1,
            "http-response-code": "200",
            "http-url": 1,
            "url-type": "GET",
            "url-path": "/",
        },
        HTTPS: {
            "https": 1,
            "web-port": 443,
            "https-expect": 1,
            "https-response-code": "200",
            "https-url": 1,
            "url-type": "GET",
            "url-path": "/",
            "disable-sslv2hello": 0
        },
        TCP: {
            "method-tcp": 1,
            "tcp-port": 80
        },
    }

    def get(self, name, **kwargs):
        return self._get(self.url_prefix + name, **kwargs)

    def _set(self, action, name, mon_method, interval, timeout, max_retries,
             method=None, url=None, expect_code=None, port=None, ipv4=None, update=False,
             **kwargs):
        params = {
            "monitor": {
                "name": name,
                "retry": int(max_retries),
                "interval": int(interval),
                "timeout": int(timeout),
                "method": {
                    mon_method: self._method_objects[mon_method]
                },
                "override-ipv4": ipv4
            }
        }
        if method:
            params['monitor']['method'][mon_method]['url-type'] = method
        if url:
            params['monitor']['method'][mon_method]['url-path'] = url
        if expect_code:
            k = "%s-response-code" % mon_method
            params['monitor']['method'][mon_method][k] = str(expect_code)
        if port:
            if mon_method == self.HTTPS:
                k = 'web-port'
            else:
                k = '%s-port' % mon_method
            params['monitor']['method'][mon_method][k] = int(port)
            params['monitor']['override-port'] = int(port)

        # TODO(mdurrant) : Might have to get tricky with JSON structures
        # ... due to 'mon_method' stuff.
        config_defaults = kwargs.get("config_defaults")

        if config_defaults:
            for k, v in six.iteritems(config_defaults):
                params['monitor'][k] = v

        if update:
            action += name
        return self._post(action, params, **kwargs)

    def create(self, name, mon_type, interval, timeout, max_retries,
               method=None, url=None, expect_code=None, port=None, ipv4=None, **kwargs):
        try:
            self.get(name)
        except acos_errors.NotFound:
            pass
        else:
            raise acos_errors.Exists()

        return self._set(self.url_prefix, name, mon_type, interval, timeout,
                         max_retries, method, url, expect_code, port, ipv4, update=False,
                         **kwargs)

    def update(self, name, mon_type, interval, timeout, max_retries,
               method=None, url=None, expect_code=None, port=None, ipv4=None, **kwargs):
        self.get(name)  # We want a NotFound if it does not exist
        return self._set(self.url_prefix, name, mon_type, interval, timeout,
                         max_retries, method, url, expect_code, port, ipv4, update=True,
                         **kwargs)

    def delete(self, name):
        return self._delete(self.url_prefix + name)



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class DNS(base.BaseV30):
    url_prefix = "/ip/dns/"

    def _set_dns(self, precedence, addr):
        addr_spec = {}

        if ':' in addr:
            addr_spec['ip-v6-addr'] = addr
        else:
            addr_spec['ip-v4-addr'] = addr

        payload = {precedence: addr_spec}

        return self._post(self.url_prefix + precedence, payload)

    def _set_suffix(self, suffix):

        payload = {'suffix': {'domain-name': suffix}}

        return self._post(self.url_prefix + 'suffix', payload)

    def set(self, primary=None, secondary=None, suffix=None):
        if primary is not None:
            self._set_dns('primary', primary)

        if secondary is not None:
            self._set_dns('secondary', secondary)

        if suffix is not None:
            self._set_suffix(suffix)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v21 import base


class BasePersistence(base.BaseV21):

    def __init__(self, client):
        super(BasePersistence, self).__init__(client)
        self.prefix = "slb.template.%s_persistence" % self.pers_type

    def get(self, name, **kwargs):
        return self._post(("%s.search" % self.prefix), {'name': name},
                          **kwargs)

    def exists(self, name, **kwargs):
        try:
            self.get(name, **kwargs)
            return True
        except acos_errors.NotFound:
            return False

    def create(self, name, **kwargs):
        self._post(("%s.create" % self.prefix), self.get_params(name),
                   **kwargs)

    def delete(self, name, **kwargs):
        self._post(("%s.delete" % self.prefix), {'name': name}, **kwargs)


class CookiePersistence(BasePersistence):

    def __init__(self, client):
        self.pers_type = 'cookie'
        super(CookiePersistence, self).__init__(client)

    def get_params(self, name):
        return {
            "cookie_persistence_template": {
                "name": name
            }
        }


class SourceIpPersistence(BasePersistence):

    def __init__(self, client):
        self.pers_type = 'src_ip'
        super(SourceIpPersistence, self).__init__(client)

    def get_params(self, name):
        return {
            "src_ip_persistence_template": {
                "name": name
            }
        }



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client import client
import acos_client.errors as acos_errors
import json
import responses


HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/services/rest/v2.1/?format=json&method='.format(HOSTNAME)
AUTH_URL = '{}authenticate'.format(BASE_URL)
SESSION_ID = 'foobar'
CREATE_URL = '{}slb.virtual_server.vport.create&session_id={}'.format(BASE_URL, SESSION_ID)
UPDATE_URL = '{}slb.virtual_server.vport.update&session_id={}'.format(BASE_URL, SESSION_ID)
DELETE_URL = '{}slb.virtual_server.vport.delete&session_id={}'.format(BASE_URL, SESSION_ID)
SEARCH_URL = '{}slb.virtual_server.search&session_id={}'.format(BASE_URL, SESSION_ID)


class TestVirtualPort(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')
        self.maxDiff = None

    @responses.activate
    def test_virtual_port_create_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            "name": "test1",
            "vport": {
                "name": "test1_VPORT",
                "service_group": "pool1",
                "protocol": 11,
                "port": 80,
                "status": 1
            }
        }

        resp = self.client.slb.virtual_server.vport.create(
            'test1', 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80',
            service_group_name='pool1'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_create_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            'name': 'test1',
            'vport': {
                'cookie_persistence_template': 'test_c_pers_template',
                'ip_in_ip': 1,
                'name': 'test1_VPORT',
                'port': 80,
                'protocol': 11,
                'service_group': 'pool1',
                'source_ip_persistence_template': 'test_s_pers_template',
                'ha-conn-mirror': 1,
                'no-dest-nat': 1,
                'conn-limit': 50000,
                'source_nat': 'test_nat_pool',
                'source_nat_auto': 1,
                'status': 1,
                'tcp_template': 'test_tcp_template',
                'udp_template': 'test_udp_template'
            }
        }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name='test1',
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1406, "msg": "The virtual port already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
        params = {
            "name": "test1",
            "vport": {
                "name": "test1_VPORT",
                "service_group": "pool1",
                "protocol": 11,
                "port": 80,
                "status": 1
            }
        }

        with self.assertRaises(acos_errors.Exists):
            self.client.slb.virtual_server.vport.create(
                'test1', 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80',
                service_group_name='pool1'
            )

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_update_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            "name": "test1",
            "vport": {
                "name": "test1_VPORT",
                "service_group": "pool1",
                "protocol": 11,
                "port": 80,
                "status": 1
            }
        }

        resp = self.client.slb.virtual_server.vport.update(
            'test1', 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80',
            service_group_name='pool1'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_update_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, UPDATE_URL, json=json_response, status=200)
        params = {
            'name': 'test1',
            'vport': {
                'cookie_persistence_template': 'test_c_pers_template',
                'ip_in_ip': 1,
                'name': 'test1_VPORT',
                'port': 80,
                'protocol': 11,
                'service_group': 'pool1',
                'source_ip_persistence_template': 'test_s_pers_template',
                'source_nat': 'test_nat_pool',
                'source_nat_auto': 1,
                'status': 1,
                'ha-conn-mirror': 1,
                'no-dest-nat': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'udp_template': 'test_udp_template'
            }
        }

        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name='test1',
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, UPDATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {"foo": "bar"}
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)
        params = {
            "name": "test1",
            "vport": {
                "name": "test1_VPORT",
                "protocol": 11,
                "port": 80,
            }
        }

        resp = self.client.slb.virtual_server.vport.delete(
            'test1', 'test1_VPORT', self.client.slb.virtual_server.vport.HTTP, '80'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1043, "msg": "Can not find the virtual server port"}}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)
        params = {
            "name": "test1",
            "vport": {
                "name": "test1_VPORT",
                "protocol": 11,
                "port": 80,
            }
        }

        resp = self.client.slb.virtual_server.vport.delete(
            'test1', 'test1_VPORT', self.client.slb.virtual_server.vport.HTTP, '80'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v21.slb.virtual_port.VirtualPort._post')
    @responses.activate
    def test_virtual_port_search(self, mocked__post):
        mocked__post.return_value = {
            'virtual_server': {
                'name': 'test1',
                'vport_list': [
                    {
                        'protocol': 12,
                        'port': 443,
                        'name': 'test2_VPORT',
                    },
                    {
                        'protocol': 11,
                        'port': 80,
                        'name': 'test1_VPORT',
                    }
                ]
            }
        }
        json_response = {
            'protocol': 11,
            'port': 80,
            'name': 'test1_VPORT',
        }

        resp = self.client.slb.virtual_server.vport.get(
            'test1', 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80'
        )

        self.assertEqual(resp, json_response)

    @responses.activate
    def test_virtual_port_search_virtual_server_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': SESSION_ID})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1043, "msg": "Can not find the virtual server port"}}
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.virtual_server.vport.get(
                'test1', 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80'
            )

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base
from acos_client.v21.slb.port import Port


class Server(base.BaseV21):

    def get(self, name, **kwargs):
        return self._post("slb.server.search", {'name': name}, **kwargs)

    def create(self, name, ip_address, status=1, **kwargs):
        params = {
            "server": {
                "name": name,
                "host": ip_address,
                "status": status,
                "conn_resume": kwargs.get("conn_resume", 0),
                "conn_limit": kwargs.get("conn_limit", 8000000),
            }
        }
        self._post("slb.server.create", params, **kwargs)

    def update(self, name, ip_address, status=1, **kwargs):
        params = {
            "server": {
                "name": name,
                "host": ip_address,
                "status": status,
                "conn_resume": kwargs.get("conn_resume", 0),
                "conn_limit": kwargs.get("conn_limit", 8000000),
            }
        }
        self._post("slb.server.update", params, **kwargs)

    def fetchStatistics(self, name, **kwargs):
        return self._post("slb.server.fetchStatistics", {"name": name},
                          **kwargs)

    def delete(self, name, **kwargs):
        self._post("slb.server.delete", {"server": {"name": name}}, **kwargs)

    def all(self, **kwargs):
        return self._get('slb.server.getAll', **kwargs)

    def all_delete(self, **kwargs):
        self._get('slb.server.deleteAll', **kwargs)

    def stats(self, name, **kwargs):
        return self._post("slb.server.fetchStatistics",
                          {"server": {"name": name}}, **kwargs)

    def all_stats(self, **kwargs):
        return self._get('fetchAllStatistics', **kwargs)

    @property
    def port(self):
        return Port(self.client)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class Nat(base.BaseV21):
    @property
    def pool(self):
        return self.Pool(self.client)

    class Pool(base.BaseV21):
        def _set(self, action, name, start_ip, end_ip, mask, **kwargs):
            params = {
                'name': name,
                'start_ip_addr': start_ip,
                'end_ip_addr': end_ip,
                'netmask': mask,
            }
            return self._post(action, params, **kwargs)

        def all(self):
            return self._get('nat.pool.getAll')

        def create(self, name, start_ip, end_ip, mask, **kwargs):
            return self._set('nat.pool.create', name, start_ip, end_ip, mask,
                             **kwargs)

        def update(self, name, start_ip, end_ip, mask, **kwargs):
            return self._set('nat.pool.create', name, start_ip, end_ip, mask,
                             **kwargs)

        def delete(self, name, **kwargs):
            return self._post('nat.pool.delete', {"name": name}, **kwargs)

        def stats(self, name, **kwargs):
            return self._post('nat.pool.fetchStatistics', {"name": name},
                              **kwargs)

        def all_stats(self, **kwargs):
            return self._get('nat.pool.fetchALLStatistics', **kwargs)



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

import six
import time

from acos_client import errors as acos_errors
from acos_client.v30 import base

INTERVAL_MONTHLY = 1
INTERVAL_DAILY = 2
INTERVAL_HOURLY = 3

DEFAULT_LICENSE_PORT = 443


class LicenseManager(base.BaseV30):

    url_base = "/license-manager"

    def create(self, host_list=[], serial=None, instance_name=None, use_mgmt_port=False,
               interval=None, bandwidth_base=None, bandwidth_unrestricted=None):
        """Creates a license manager entry

        Keyword arguments:
        instance_name -- license manager instance name
        host_list -- list(dict) a list of dictionaries of the format:
            {'ip': '127.0.0.1', 'port': 443}
        serial - (str) appliance serial number
        use_mgmt_port - (bool) use management for license interactions
        interval - (int) 1=Monthly, 2=Daily, 3=Hourly
        bandwidth_base - (int) Configure feature bandwidth base (Mb)
            Valid range - 10-102400
        bandwidth_unrestricted - (bool) Set the bandwidth to maximum
        """
        payload = self._build_payload(host_list=host_list, serial=serial,
                                      instance_name=instance_name,
                                      use_mgmt_port=use_mgmt_port,
                                      interval=interval, bandwidth_base=bandwidth_base,
                                      bandwidth_unrestricted=bandwidth_unrestricted)
        return self._post(self.url_base, payload)

    def update(self, host_list=[], serial=None, instance_name=None, use_mgmt_port=False,
               interval=None, bandwidth_base=None, bandwidth_unrestricted=None):
        """Update a license manager entry

        Keyword arguments:
        instance_name -- license manager instance name
        host_list -- list(dict) a list of dictionaries of the format:
            {'ip': '127.0.0.1', 'port': 443}
        serial - (str) appliance serial number
        use_mgmt_port - (bool) use management for license interactions
        interval - (int) 1=Monthly, 2=Daily, 3=Hourly
        bandwidth_base - (int) Configure feature bandwidth base (Mb)
            Valid range - 10-102400
        bandwidth_unrestricted - (bool) Set the bandwidth to maximum
        """

        return self.create(host_list=host_list, serial=serial, instance_name=instance_name,
                           use_mgmt_port=use_mgmt_port,
                           interval=interval, bandwidth_base=bandwidth_base,
                           bandwidth_unrestricted=bandwidth_unrestricted)

    def get(self):
        return self._get(self.url_base)

    def connect(self, connect=False):
        url = self.url_base + "/connect"
        payload = {
            "connect": {
                "connect": 1 if connect else 0,
            }
        }
        return self._post(url, payload)

    def _build_payload(self, host_list=[], serial=None, instance_name=None, use_mgmt_port=False,
                       interval=None, bandwidth_base=None, bandwidth_unrestricted=None):
        rv = {"license-manager": {}}
        if host_list:
            for x in host_list:
                hosts = []
                hosts.append(self._build_host_entry(x["ip"], x.get("port")))

            rv["license-manager"]["host-list"] = hosts

        self._set_if_set(instance_name, rv["license-manager"], "instance-name")
        self._set_if_set(serial, rv["license-manager"], "sn")
        self._set_if_set(use_mgmt_port, rv["license-manager"], "use-mgmt-port")
        self._set_if_set(interval, rv["license-manager"], "interval")
        self._set_if_set(bandwidth_base, rv["license-manager"], "bandwidth-base")
        self._set_if_set(bandwidth_unrestricted, rv["license-manager"], "bandwidth-unrestricted")

        return rv

    def _build_host_entry(self, ip, port=DEFAULT_LICENSE_PORT):
        return {"host-ipv4": ip, "port": port or DEFAULT_LICENSE_PORT}

    def _set_if_set(self, src, dest, dest_key):
        if src is not None:
            dest[dest_key] = src

    #
    # The method below, 'paygo', works more reliably than the axapi above.
    # But be forewarned, looking further might cause blindness.
    #

    def _paygo_setup(self, llp_hosts=[], sn=None, instance_name=None,
                     use_mgmt_port=False, interval=None, bandwidth_base=None):
        url = "/clideploy/"
        commands = []
        if use_mgmt_port:
            commands += ["license-manager use-mgmt-port"]
        for host in llp_hosts:
            commands += ["license-manager host %s" % host]
        commands += [
            "license-manager sn %s" % sn,
            "license-manager interval %s" % interval,
            "license-manager instance-name %s" % instance_name,
            "license-manager bandwidth-base %s" % bandwidth_base,
        ]
        payload = {
            "commandlist": commands
        }
        self._post(url, payload)

    def _paygo_connect(self):
        url = "/clideploy/"
        payload = {
            "commandlist": [
                "license-manager connect"
            ]
        }

        # There is some lag between the setup call above and being able
        # to successfully retrieve a license.

        for i in six.moves.range(0, 60):
            try:
                return self._post(url, payload)
            except acos_errors.ACOSException as e:
                if 'Invalid message' in str(e):
                    time.sleep(5)
                    continue
                raise e

    def paygo(self, llp_hosts=[], sn=None, instance_name=None,
              use_mgmt_port=False, interval=None, bandwidth_base=None):

        for i in six.moves.range(0, 4):
            self._paygo_setup(llp_hosts, sn, instance_name, use_mgmt_port,
                              interval, bandwidth_base)
            try:
                self._paygo_connect()
            except acos_errors.ACOSException as e:
                if 'Communication error' in str(e):
                    self.client.session.close()
                    time.sleep(5)
                    continue
                raise e



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30.vrrpa import blade_params


class TestBlade(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = blade_params.BladeParameters(self.client)
        self.url_prefix = "/axapi/v3/vrrp-a/vrid/{0}/blade-parameters"

    def _expected_payload(self, priority=None, interface=None, gateway=None):
        rv = {'blade-parameters': {}}
        if priority:
            rv['blade-parameters']['priority'] = priority

        if interface:
            rv['blade-parameters']['tracking-options'] = interface

        if gateway:
            if rv['blade-parameters'].get('tracking-options'):
                rv['blade-parameters']['tracking-options'].update(gateway)
            else:
                rv['blade-parameters']['tracking-options'] = gateway

        return rv

    def _build_interface(self, ethernet=1, priority_cost=1):
        rv = {
            'interface': [{
                'ethernet': ethernet,
                'priority-cost': priority_cost
            }]
        }
        return rv

    def _build_ipv4gateway(self, ip_address, priority_cost=1):
        rv = {
            'gateway': {
                'ipv4-gateway-list': [{
                    'ip-address': ip_address,
                    'priority-cost': priority_cost
                }],
                'ipv6-gateway-list': []
            }
        }
        return rv

    def _build_ipv6gateway(self, ip_address, priority_cost=1):
        rv = {
            'gateway': {
                'ipv6-gateway-list': [{
                    'ip-address': ip_address,
                    'priority-cost': priority_cost
                }],
                'ipv4-gateway-list': []
            }
        }
        return rv

    def test_blade_get(self):
        self.target.get(0)
        self.client.http.request.assert_called_with("GET", self.url_prefix.format(0), {}, mock.ANY)

    def test_blade_create(self):
        self.target.create(4)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(), mock.ANY)

    def test_blade_create_priority(self):
        self.target.create(4, 122)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(122), mock.ANY)

    def test_blade_create_interface(self):
        interface = self._build_interface()

        self.target.add_interface()
        self.target.create(4)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(interface=interface), mock.ANY)

    def test_blade_create_gateway(self):
        gateway = self._build_ipv4gateway('1.1.1.1')

        self.target.add_ipv4gateway('1.1.1.1')
        self.target.create(4)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(gateway=gateway), mock.ANY)

    def test_blade_create_gateway_ipv6(self):
        gateway = self._build_ipv6gateway('1.1.1.1')

        self.target.add_ipv6gateway('1.1.1.1')
        self.target.create(4)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(gateway=gateway), mock.ANY)

    def test_blade_create_interface_gateway(self):
        interface = self._build_interface()
        gateway = self._build_ipv4gateway('1.1.1.1')

        self.target.add_interface()
        self.target.add_ipv4gateway('1.1.1.1')
        self.target.create(4)
        self.client.http.request.assert_called_with(
            "POST", self.url_prefix.format(4),
            self._expected_payload(interface=interface, gateway=gateway), mock.ANY)

    def test_blade_update(self):
        self.target.update(4)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(), mock.ANY)

    def test_blade_update_priority(self):
        self.target.update(4, 122)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(122), mock.ANY)

    def test_blade_update_interface(self):
        interface = self._build_interface()

        self.target.add_interface()
        self.target.update(4)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(interface=interface), mock.ANY)

    def test_blade_update_gateway(self):
        gateway = self._build_ipv4gateway('1.1.1.1')

        self.target.add_ipv4gateway('1.1.1.1')
        self.target.update(4)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(gateway=gateway), mock.ANY)

    def test_blade_update_gateway_ipv6(self):
        gateway = self._build_ipv6gateway('1.1.1.1')

        self.target.add_ipv6gateway('1.1.1.1')
        self.target.update(4)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(gateway=gateway), mock.ANY)

    def test_blade_update_interface_gateway(self):
        interface = self._build_interface()
        gateway = self._build_ipv4gateway('1.1.1.1')

        self.target.add_interface()
        self.target.add_ipv4gateway('1.1.1.1')
        self.target.update(4)
        self.client.http.request.assert_called_with(
            "PUT", self.url_prefix.format(4),
            self._expected_payload(interface=interface, gateway=gateway), mock.ANY)



# Copyright 2016, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v30 import base


class Interface(base.BaseV30):
    iftype = "interface"
    url_prefix = "/interface/"

    def __init__(self, client):
        super(Interface, self).__init__(client)

    def _url_from_ifnum(self, ifnum=None):
        return self.url_prefix + self._ifnum_to_str(ifnum)

    def _ifnum_to_str(self, ifnum=None):
        return str(ifnum if ifnum else "")

    def _build_payload(self, ifnum=None, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
                       speed="auto", **kwargs):
        # TODO(mdurrant) - Check ip/netmask for validity.
        rv = {
            "interface": {
                "ip": {
                }
            }
        }

        if ifnum:
            rv["interface"]["ifnum"] = ifnum
        if ip_address and not dhcp:
            rv["interface"]["ip"]["address-list"] = [
                {"ipv4-address": ip_address, "ipv4-netmask": ip_netmask}
            ]
        else:
            rv["interface"]["ip"]["dhcp"] = 1 if dhcp is True else 0

        if enable is not None:
            rv["interface"]["action"] = "enable" if enable else "disable"

        return rv

    def get_list(self):
        return self._get(self.url_prefix)

    def get(self, ifnum=None):
        return self._get(self._url_from_ifnum(ifnum))

    def exists(self, ifnum=None):
        try:
            self.get(ifnum)
            return True
        except acos_errors.NotFound:
            return False

    def delete(self, ifnum):
        url = self.url_prefix + self._ifnum_to_str(ifnum)
        return self._delete(url)

    def create(self, ifnum, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
               speed="auto"):

        payload = self._build_payload(ifnum=ifnum, ip_address=ip_address, ip_netmask=ip_netmask,
                                      dhcp=dhcp, enable=enable, speed=speed)
        return self._post(self.url_prefix, payload)

    def update(self, ifnum, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
               speed="auto"):
        payload = self._build_payload(ifnum=ifnum, ip_address=ip_address, ip_netmask=ip_netmask,
                                      dhcp=dhcp, enable=enable, speed=speed)
        return self._post(self.url_prefix + self._ifnum_to_str(ifnum),
                          payload)

    def get_oper(self, ifnum):
        url = "{0}{1}/oper".format(self.url_prefix, ifnum)
        return self._get(url)

    @property
    def ethernet(self):
        return EthernetInterface(self.client)

    @property
    def management(self):
        return ManagementInterface(self.client)

    @property
    def lif(self):
        return LogicalInterface(self.client)

    @property
    def ve(self):
        return VirtualEthernet(self.client)


class EthernetInterface(Interface):
    def __init__(self, client):
        super(EthernetInterface, self).__init__(client)
        self.iftype = "ethernet"
        self.url_prefix = "{0}{1}/".format(self.url_prefix, self.iftype)

    def _build_payload(self, **kwargs):
        rv = super(EthernetInterface, self)._build_payload(**kwargs)
        # Allows us to use the Interface class for ethernet ifs
        rv[self.iftype] = rv.pop("interface")
        return rv


class ManagementInterface(Interface):
    def __init__(self, client):
        super(ManagementInterface, self).__init__(client)
        self.iftype = "management"
        self.url_prefix = "{0}{1}/".format(self.url_prefix, self.iftype)

    def _build_payload(self, **kwargs):
        rv = super(ManagementInterface, self)._build_payload(**kwargs)
        # Allows us to use the Interface class for management ifs
        rv[self.iftype] = rv.pop("interface")
        default_gateway = kwargs.get("default_gateway")
        if default_gateway:
            rv[self.iftype]["ip"]["default-gateway"] = default_gateway

        return rv

    def create(self, ifnum=None, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
               speed="auto", default_gateway=None):
        payload = self._build_payload(ifnum=ifnum, ip_address=ip_address, ip_netmask=ip_netmask,
                                      dhcp=dhcp, enable=enable, speed=speed,
                                      default_gateway=default_gateway)
        return self._post(self.url_prefix,
                          payload)

    def update(self, ifnum=None, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
               speed="auto", default_gateway=None):
        payload = self._build_payload(ifnum=ifnum, ip_address=ip_address, ip_netmask=ip_netmask,
                                      dhcp=dhcp, enable=enable, speed=speed,
                                      default_gateway=default_gateway)
        return self._post(self.url_prefix + self._ifnum_to_str(ifnum),
                          payload)


class LogicalInterface(Interface):
    def __init__(self, client):
        super(LogicalInterface, self).__init__(client)
        self.iftype = "lif"
        self.url_prefix = "{0}{1}/".format(self.url_prefix, self.iftype)

    def create(self, ifnum=None, ip_address=None, ip_netmask=None, dhcp=False, enable=None,
               speed="auto", default_gateway=None):
        payload = self._build_payload(ifnum=ifnum, ip_address=ip_address, ip_netmask=ip_netmask,
                                      dhcp=dhcp, enable=enable, speed=speed,
                                      default_gateway=default_gateway)
        return self._post(self.url_prefix, payload)

    def _build_payload(self, ifnum=None, ip_address=None, ip_netmask=None, dhcp=False,
                       enable=None, speed="auto", default_gateway=None):
        # TODO(mdurrant) - Check ip/netmask for validity.
        rv = {
            self.iftype: {
                "ip": {
                }
            }
        }

        if ifnum:
            rv[self.iftype]["ifnum"] = ifnum
        if ip_address and not dhcp:
            rv[self.iftype]["ip"]["address-list"] = [
                {"ipv4-address": ip_address, "ipv4-netmask": ip_netmask}
            ]
        else:
            rv[self.iftype]["ip"]["dhcp"] = 1 if dhcp is True else 0

        if enable is not None:
            rv[self.iftype]["action"] = "enable" if enable else "disable"

        return rv


class VirtualEthernet(Interface):
    def __init__(self, client):
        super(VirtualEthernet, self).__init__(client)
        self.iftype = "ve"
        self.url_prefix = "{0}{1}/".format(self.url_prefix, self.iftype)

    def _build_payload(self, **kwargs):
        # we need to deal with VLAN stuff here
        rv = super(VirtualEthernet, self)._build_payload(**kwargs)
        rv[self.iftype] = rv.pop("interface")

        return rv
